use std::{io::stdin, str::FromStr};
use anyhow::{anyhow, Result, Error};
use itertools::Itertools;

struct Card {
    id: u8,
    drawn: [bool; 100],
    chosen:[bool; 100],
    wins: u8,
}

impl Card {
    fn value(&self) -> usize {
        self.wins
            .checked_sub(1)
            .map(|e| 2_usize.pow(e as u32))
            .unwrap_or(0)
    }
}

impl FromStr for Card {
    type Err = Error;
    fn from_str(s: &str) -> Result<Self> {
        let (id_s, drawn_s, chosen_s) = s.split(&[':','|'])
            .collect_tuple()
            .ok_or_else(||anyhow!("Bad line format {s}"))?;
        let id = id_s.strip_prefix("Card ")
                .ok_or_else(|| anyhow!("Bad header {id_s}"))?
                .trim()
                .parse()?;

        let mut drawn = [false; 100];
        let mut chosen = [false; 100];
        let mut wins = 0;

        for d in drawn_s.trim().split_whitespace() {
            drawn[d.parse::<usize>()?] = true;
        }
        for c in chosen_s.trim().split_whitespace() {
            let c: usize = c.parse()?;
            chosen[c] = true;
            if drawn[c] { wins += 1 }
        }

        Ok(Card { id, drawn, chosen, wins })
    }
}

fn read() -> Result<Vec<Card>> {
    stdin()
        .lines()
        .map(|s| s?.parse())
        .collect()
}

fn main() -> Result<()> {
    let cards = read()?;
    let score1: usize = cards.iter().map(Card::value).sum();
    println!("score is {score1}");

    let mut counts: Vec<usize> = cards.iter().map(|_| 1).collect();
    for i in 0..counts.len() {
        let amount = counts[i];
        let wins = cards[i].wins as usize;
        for other in &mut counts[i+1..][..wins] {
            *other += amount;
        }
    }
    let total: usize = counts.iter().sum();
    println!("Total cards {total}");
    Ok(())
}
