use std::{collections::BTreeMap, io::stdin, ops::Mul};
use anyhow::{Result, bail};

mod parse;

/// Stores a map as a series of contiguous offset ranges.
#[derive(Debug)]
struct Map {
    from: String,
    to: String,
    ranges: BTreeMap<usize, usize>
}

impl Map {

    fn identity() -> Self {
        let ranges = BTreeMap::from_iter(
            Some((0,0)).into_iter()
        );

        Map {
            from: "-".to_owned(),
            to: "seed".to_owned(),
            ranges,
        }
    }

    fn follow(&self, src: usize) -> usize {
        self.ranges
            .range(..src+1)
            .last().map(|(s,d)| src-s+d)
            .unwrap_or(src)
    }
}

impl Mul<&Map> for &Map {
    type Output = Option<Map>;

    fn mul(self, rhs: &Map) -> Self::Output {
        let mut cur = self.ranges.iter();
        let mut next = self.ranges.keys();
        next.next();

        let mut result = BTreeMap::new();

        while let Some((&base, &tgt)) = cur.next() {
            let len = next.next().map(|n| n - base);
            let tgt_range = match len {
                Some(len) => rhs.ranges.range(tgt..tgt+len),
                None => rhs.ranges.range(tgt..)
            };

            for (&rbase, &rtgt) in tgt_range {
                
            }

        }
        Some(Map { ranges: result, from: self.from.to_owned(), to: rhs.to.to_owned() }) 

    }
} 

fn main() -> Result<()> {
    let stdin = stdin().lock();
    let (seeds, maps) = parse::input(stdin)?;
    println!("{maps:?}");

    let mut step = "seed";
    for map in &maps {
        if &map.from != step { 
            bail!("incorrect map order, got {map:?}, expected {step}");
        }
        step = &map.to;
    }

    let mut best = None;

    for &seed in &seeds {
        let mut x = seed;
        
        for map in &maps {
            x = map.follow(x);
        }


        if best.map(|old| old > x).unwrap_or(true) {
            best = Some(x)
        }
    }

    println!("Best: {best:?}");

    Ok(())
}
